#include "JetCluAlgorithm.hh"
#include "MidPointAlgorithm.hh"
#include "PhysicsTower.hh"
#include "Cluster.hh"
#include <fstream>
#include <iostream>
#include <cstdio>

int main()
{
  // Define JetClu algorithm.
  double j_seedThreshold    = 1;
  double j_coneRadius       = 0.7;
  int    j_adjacencyCut     = 2;
  int    j_maxIterations    = 100;
  int    j_iratch           = 1;
  double j_overlapThreshold = 0.75;
  JetCluAlgorithm j(j_seedThreshold,j_coneRadius,j_adjacencyCut,j_maxIterations,j_iratch,j_overlapThreshold);
  //  JetCluAlgorithm j;

  // Define MidPoint algorithm.
  double m_seedThreshold    = 1;
  double m_coneRadius       = 0.7;
  double m_coneAreaFraction = 0.25;
  int    m_maxPairSize      = 2;
  int    m_maxIterations    = 100;
  double m_overlapThreshold = 0.75;
  MidPointAlgorithm m(m_seedThreshold,m_coneRadius,m_coneAreaFraction,m_maxPairSize,m_maxIterations,m_overlapThreshold);
  //  MidPointAlgorithm m;

  std::ifstream i;
  i.open("../event/event.dat");
  double px,py,pz,E;
  std::vector<PhysicsTower> towers;
  while(i >> px >> py >> pz >> E)
    towers.push_back(PhysicsTower(LorentzVector(px,py,pz,E)));

  std::vector<Cluster> j_jets;
  j.run(towers,j_jets);

  std::cout << "JetClu:" << std::endl;
  for(std::vector<Cluster>::iterator jetIter = j_jets.begin(); jetIter != j_jets.end(); jetIter++)
    printf("Et = %11.6f, eta = %9.6f, phi = %8.6f, nTowers = %3d\n",
	   jetIter->fourVector.Et(),jetIter->fourVector.eta(),jetIter->fourVector.phi(),jetIter->towerList.size());
  std::cout << std::endl;

  std::vector<Cluster> m_jets;
  m.run(towers,m_jets);

  std::cout << "MidPoint:" << std::endl;
  for(std::vector<Cluster>::iterator jetIter = m_jets.begin(); jetIter != m_jets.end(); jetIter++)
    printf("Et = %11.6f, eta = %9.6f, phi = %8.6f, nTowers = %3d\n",
	   jetIter->fourVector.Et(),jetIter->fourVector.eta(),jetIter->fourVector.phi(),jetIter->towerList.size());
  std::cout << std::endl;
}
