#include "JetCluAlgorithm.hh"
#include "MidPointAlgorithm.hh"
#include "PhysicsTower.hh"
#include "LorentzVector.hh"
#include "Cluster.hh"
#include <vector>

extern "C"
{
  void jclcpp_(double& seedThreshold,
	       double& coneRadius,
	       int& adjacencyCut,
	       int& maxIterations,
	       int& iratch,
	       double& overlapThreshold,
	       double pp[2000][4], int& np, double jp[2000][4], int jc[2000][2001], int& nj)
  {
    // Define jet algorithm.
    JetCluAlgorithm j(seedThreshold,coneRadius,adjacencyCut,maxIterations,iratch,overlapThreshold);

    // Read event.
    std::vector<PhysicsTower> particles;
    for(int i = 0; i < np; i++)
      particles.push_back(PhysicsTower(LorentzVector(pp[i][0],pp[i][1],pp[i][2],pp[i][3])));

    // Run algorithm.
    std::vector<Cluster> jets;
    j.run(particles,jets);

    // Write jet quantities into Fortran arrays.
    nj = jets.size();
    for(int i = 0; i < nj; i++){
      jp[i][0] = jets[i].fourVector.px;
      jp[i][1] = jets[i].fourVector.py;
      jp[i][2] = jets[i].fourVector.pz;
      jp[i][3] = jets[i].fourVector.E;
      jc[i][0] = 0;
      int iParticle = 0;
      for(std::vector<PhysicsTower>::iterator particleIter = particles.begin();
	  particleIter != particles.end();
	  particleIter++){
	iParticle++;
	for(std::vector<PhysicsTower>::iterator jetParticleIter = jets[i].towerList.begin();
	    jetParticleIter != jets[i].towerList.end();
	    jetParticleIter++)
	  if(particleIter->isEqual(*jetParticleIter)){
	    jc[i][0]++;
	    jc[i][jc[i][0]] = iParticle;
	  }
      }
    }
  }

  void midcpp_(double& seedThreshold,
	       double& coneRadius,
	       double& coneAreaFraction,
	       int& maxPairSize,
	       int& maxIterations,
	       double& overlapThreshold,
	       double pp[2000][4], int& np, double jp[2000][4], int jc[2000][2001], int& nj)
  {
    // Define jet algorithm.
    MidPointAlgorithm m(seedThreshold,coneRadius,coneAreaFraction,maxPairSize,maxIterations,overlapThreshold);

    // Read event.
    std::vector<PhysicsTower> particles;
    for(int i = 0; i < np; i++)
      particles.push_back(PhysicsTower(LorentzVector(pp[i][0],pp[i][1],pp[i][2],pp[i][3])));

    // Run algorithm.
    std::vector<Cluster> jets;
    m.run(particles,jets);

    // Write jet quantities into Fortran arrays.
    nj = jets.size();
    for(int i = 0; i < nj; i++){
      jp[i][0] = jets[i].fourVector.px;
      jp[i][1] = jets[i].fourVector.py;
      jp[i][2] = jets[i].fourVector.pz;
      jp[i][3] = jets[i].fourVector.E;
      jc[i][0] = 0;
      int iParticle = 0;
      for(std::vector<PhysicsTower>::iterator particleIter = particles.begin();
	  particleIter != particles.end();
	  particleIter++){
	iParticle++;
	for(std::vector<PhysicsTower>::iterator jetParticleIter = jets[i].towerList.begin();
	    jetParticleIter != jets[i].towerList.end();
	    jetParticleIter++)
	  if(particleIter->isEqual(*jetParticleIter)){
	    jc[i][0]++;
	    jc[i][jc[i][0]] = iParticle;
	  }
      }
    }
  }
}
