      program main

      implicit none

      double precision jSdThr,jConeR,jOvlThr
      integer jAdjCut,jMxIter,jIratch

      double precision mSdThr,mConeR,mCnArF,mOvlThr
      integer mMxPrS,mMxIter

      integer np,nj
      double precision pp(4,2000),jp(4,2000)
      integer jc(2001,2000)

      integer i,j
      double precision p,pt,phi
      double precision twopi
      parameter (twopi = 6.283185307)

c     Define JetClu parameters.
c     - seedThreshold:
      jSdThr  = 1
c     - coneRadius:
      jConeR  = 0.7
c     - adjacencyCut:
      jAdjCut = 2
c     - maxIterations:
      jMxIter = 100
c     - ratcheting:
      jIratch = 1
c     - overlapThreshold:
      jOvlThr = 0.5

c     Define MidPoint parameters.
c     - seedThreshold:
      mSdThr  = 1
c     - coneRadius:
      mConeR  = 0.7
c     - coneAreaFraction:
      mCnArF  = 0.25
c     - maxPairSize:
      mMxPrS  = 2
c     - maxIterations:
      mMxIter = 100
c     - overlapThreshold:
      mOvlThr = 0.75

c     Fill pp (4-vector array) and ip (number of input particles).
      open(unit=21,file='../event/event.dat',status='old')
      do 1 i = 1, 129
         read(21,'(f11.6,1x,f11.6,1x,f11.6,1x,f11.6)')
     +        pp(1,i),pp(2,i),pp(3,i),pp(4,i)
 1    continue
      close(21)
      np = 129

c     Run JetClu algorithm.

c     jp = 4-vector array of jets
c     jc(1,i) = number of particles in jet #i
c     jc(j,i) = number of particle #(j-1) in jet #i (j=2,...,jc(1,i)+1)
c     nj = number of jets

      call jclcpp(jSdThr,jConeR,jAdjCut,jMxIter,jIratch,jOvlThr,
     +            pp,np,jp,jc,nj)

c     Print (or analyze) jets.
      print*,'List of JetClu jets:'
      do 2 i = 1, nj
         p  = sqrt(jp(1,i)*jp(1,i) + jp(2,i)*jp(2,i) + jp(3,i)*jp(3,i))
         pt = sqrt(jp(1,i)*jp(1,i) + jp(2,i)*jp(2,i))
         phi = atan2(jp(2,i),jp(1,i))
         if(phi.lt.0)
     +        phi = phi + twopi
         print*,'   Et = ',jp(4,i)/p*pt,
     +        ', eta = ',0.5*log((p + jp(3,i))/(p - jp(3,i))),
     +        ', phi = ',phi
         print*,'      particles in this jet:'
         do 3 j = 2, jc(1,i) + 1
            print*,jc(j,i)
 3       continue
 2    continue

c     Run MidPoint algorithm.

      call midcpp(mSdThr,mConeR,mCnArF,mMxPrS,mMxIter,mOvlThr,
     +            pp,np,jp,jc,nj)

c     Print (or analyze) jets.
      print*,'List of MidPoint jets:'
      do 4 i = 1, nj
         p  = sqrt(jp(1,i)*jp(1,i) + jp(2,i)*jp(2,i) + jp(3,i)*jp(3,i))
         pt = sqrt(jp(1,i)*jp(1,i) + jp(2,i)*jp(2,i))
         phi = atan2(jp(2,i),jp(1,i))
         if(phi.lt.0)
     +        phi = phi + twopi
         print*,'   Et = ',jp(4,i)/p*pt,
     +        ', eta = ',0.5*log((p + jp(3,i))/(p - jp(3,i))),
     +        ', phi = ',phi
         print*,'      particles in this jet:'
         do 5 j = 2, jc(1,i) + 1
            print*,jc(j,i)
 5       continue
 4    continue

      end
