      PROGRAM HWIGPR
C---COMMON BLOCKS ARE INCLUDED AS FILE HERWIG65.INC
      INCLUDE 'HERWIG65.INC'
      INTEGER N
      EXTERNAL HWUDAT
C---MAX NUMBER OF EVENTS THIS RUN
      MAXEV=1000
C---BEAM PARTICLES
      PART1='PBAR'
      PART2='P'
C---BEAM MOMENTA
      PBEAM1=1000.
      PBEAM2=1000.
C---PROCESS
      IPROC=1500
C---INITIALISE OTHER COMMON BLOCKS
      CALL HWIGIN
C---USER CAN RESET PARAMETERS AT
C   THIS POINT, OTHERWISE DEFAULT
C   VALUES IN HWIGIN WILL BE USED.
      PTMIN=100.
C---COMPUTE PARAMETER-DEPENDENT CONSTANTS
      CALL HWUINC
C---CALL HWUSTA TO MAKE ANY PARTICLE STABLE
      CALL HWUSTA('PI0     ')
C---USER'S INITIAL CALCULATIONS
      CALL HWABEG
C---INITIALISE ELEMENTARY PROCESS
      CALL HWEINI
C---LOOP OVER EVENTS
      DO 100 N=1,MAXEV
C---INITIALISE EVENT
      CALL HWUINE
C---GENERATE HARD SUBPROCESS
      CALL HWEPRO
C---GENERATE PARTON CASCADES
      CALL HWBGEN
C---DO HEAVY OBJECT DECAYS
      CALL HWDHOB
C---DO CLUSTER FORMATION
      CALL HWCFOR
C---DO CLUSTER DECAYS
      CALL HWCDEC
C---DO UNSTABLE PARTICLE DECAYS
      CALL HWDHAD
C---DO HEAVY FLAVOUR HADRON DECAYS
      CALL HWDHVY
C---ADD SOFT UNDERLYING EVENT IF NEEDED
      CALL HWMEVT
C---FINISH EVENT
      CALL HWUFNE
C---USER'S EVENT ANALYSIS
      CALL HWANAL
  100 CONTINUE
C---TERMINATE ELEMENTARY PROCESS
      CALL HWEFIN
C---USER'S TERMINAL CALCULATIONS
      CALL HWAEND
      STOP
      END
C----------------------------------------------------------------------
      SUBROUTINE HWABEG
C     USER'S ROUTINE FOR INITIALIZATION
C----------------------------------------------------------------------
      END
C----------------------------------------------------------------------
      SUBROUTINE HWAEND
C     USER'S ROUTINE FOR TERMINAL CALCULATIONS, HISTOGRAM OUTPUT, ETC
C----------------------------------------------------------------------
      END
C----------------------------------------------------------------------
      SUBROUTINE HWANAL
C     USER'S ROUTINE TO ANALYSE DATA FROM EVENT
C----------------------------------------------------------------------
      INCLUDE 'HERWIG65.INC'

      double precision jSdThr,jConeR,jOvlThr
      integer jAdjCut,jMxIter,jIratch

      double precision mSdThr,mConeR,mCnArF,mOvlThr
      integer mMxPrS,mMxIter

      integer np,njj,njm
      double precision pp(4,2000),jjp(4,2000),jmp(4,2000)
      integer jjc(2001,2000),jmc(2001,2000)

      integer i,j
      double precision p,pt,phi
      double precision twopi
      parameter (twopi = 6.283185307)

c     Define JetClu parameters.
c     - seedThreshold:
      jSdThr  = 1
c     - coneRadius:
      jConeR  = 0.7
c     - adjacencyCut:
      jAdjCut = 2
c     - maxIterations:
      jMxIter = 100
c     - ratcheting:
      jIratch = 1
c     - overlapThreshold:
      jOvlThr = 0.75

c     Define MidPoint parameters.
c     - seedThreshold:
      mSdThr  = 1
c     - coneRadius:
      mConeR  = 0.7
c     - coneAreaFraction:
      mCnArF  = 0.25
c     - maxPairSize:
      mMxPrS  = 2
c     - maxIterations:
      mMxIter = 100
c     - overlapThreshold:
      mOvlThr = 0.75

c     Fill pp (4-vector array) and np (number of input particles).
      np = 0
      do 1 i = 1, nhep
         if(isthep(i).eq.1) then
            np = np + 1
            pp(1,np) = phep(1,i)
            pp(2,np) = phep(2,i)
            pp(3,np) = phep(3,i)
            pp(4,np) = phep(4,i)
         endif
 1    continue

c     Run JetClu and MidPoint algorithms.

c     j(j/m)p = 4-vector array of jets
c     j(j/m)c(1,i) = number of particles in jet #i
c     j(j/m)c(j,i) = number of particle #(j-1) in jet #i (j=2,...,jc(1,i)+1)
c     nj(j/m) = number of jets

      call jclcpp(jSdThr,jConeR,jAdjCut,jMxIter,jIratch,jOvlThr,
     +            pp,np,jjp,jjc,njj)
      call midcpp(mSdThr,mConeR,mCnArF,mMxPrS,mMxIter,mOvlThr,
     +            pp,np,jmp,jmc,njm)

c     Print (or analyze) JetClu jets.
      print*,'List of JetClu jets:'
      do 2 i = 1, njj
         p  = sqrt(jjp(1,i)*jjp(1,i) + jjp(2,i)*jjp(2,i) +
     +             jjp(3,i)*jjp(3,i))
         pt = sqrt(jjp(1,i)*jjp(1,i) + jjp(2,i)*jjp(2,i))
         phi = atan2(jjp(2,i),jjp(1,i))
         if(phi.lt.0)
     +        phi = phi + twopi
         print*,'   Et = ',jjp(4,i)/p*pt,
     +        ', eta = ',0.5*log((p + jjp(3,i))/(p - jjp(3,i))),
     +        ', phi = ',phi
c         print*,'      particles in this jet:'
c         do 3 j = 2, jjc(1,i) + 1
c            print*,jjc(j,i)
c 3       continue
 2    continue

c     Print (or analyze) MidPoint jets.
      print*,'List of MidPoint jets:'
      do 4 i = 1, njm
         p  = sqrt(jmp(1,i)*jmp(1,i) + jmp(2,i)*jmp(2,i) +
     +             jmp(3,i)*jmp(3,i))
         pt = sqrt(jmp(1,i)*jmp(1,i) + jmp(2,i)*jmp(2,i))
         phi = atan2(jmp(2,i),jmp(1,i))
         if(phi.lt.0)
     +        phi = phi + twopi
         print*,'   Et = ',jmp(4,i)/p*pt,
     +        ', eta = ',0.5*log((p + jmp(3,i))/(p - jmp(3,i))),
     +        ', phi = ',phi
c         print*,'      particles in this jet:'
c         do 5 j = 2, jmc(1,i) + 1
c            print*,jmc(j,i)
c 5       continue
 4    continue

      END
C----------------------------------------------------------------------
